@@ -1,5 +1,5 @@
 package CGI::Builder::Session ;
-$VERSION = 1.23 ;
+$VERSION = 1.24 ;
 
 # This file uses the "Perlish" coding style
 # please read http://perl.4pro.net/perlish_coding_style.html
@@ -38,6 +38,14 @@ $VERSION = 1.23 ;
                                          }
                                  )
    ; $s->cs_init($cso)
+   ; if ($cso->is_new)
+      { if ( my $new_session_init = $$args{new_session_init} )
+         { $s->$new_session_init($cso)
+         }
+      }
+   ; if ( my $session_init = $$args{session_init} )
+      { $s->$session_init($cso)
+      }
    ; $cso
    }
    
@@ -67,7 +75,7 @@ __END__
 
 CGI::Builder::Session - CGI::Builder and CGI::Session integration
 
-=head1 VERSION 1.23
+=head1 VERSION 1.24
 
 The latest versions changes are reported in the F<Changes> file in this distribution. To have the complete list of all the extensions of the CBF, see L<CGI::Builder/"Extensions List">
 
@@ -109,22 +117,10 @@ From the directory where this file is located, type:
 
 =head1 DESCRIPTION
 
+B<Note>: You should know L<CGI::Builder>.
+
 This module transparently integrates C<CGI::Builder> and C<CGI::Session> in a very handy and flexible framework that can save you some coding. It provides you a mostly automatic and ready to use CGI::Session object (C<cs> property) useful to maintain the state of your application between requests. Please refer to L<CGI::Session> for more documentation about sessions.
 
-=head2 Useful links
-
-=over
-
-=item *
-
-A simple and useful navigation system between the various CBF extensions is available at this URL: L<http://perl.4pro.net>
-
-=item *
-
-More examples and more practical topics are available in the mailing list at this URL: L<http://lists.sourceforge.net/lists/listinfo/cgi-builder-users>
-
-=back
-
 =head2 How it works
 
 This extension creates a CGI::Session object automatically, using the old session id if it is found as a cookie or as a query param. If no session id is found, it creates a new session and automatically adds a session id cookie and a session id param that will be automatically used to send the id to the client
@@ -184,6 +180,27 @@ Data Source Name param This argument is referring to the File DSN, and is set by
 
     { Directory => File::Spec->tmpdir }
 
+=item * new_session_init
+
+A method name or a CODE reference which will be executed only if the session is new (i.e. if it has been created in the current request). The sub will receive the WebApp object in $_[0], and the session object in $_[1]. You can use it to add options to the session object:
+
+   new_session_init => 'any_method'
+   new_session_init => \&any_sub
+   new_session_init => sub{ ... }
+
+A tipical usage could be:
+
+   new_session_init => sub{ my ($s, $session) = @_ ;
+                            $session->expires("+1y"); }
+
+=item * session_init
+
+A method name or a CODE reference which will be executed each time the session is created and regardless the session is new or not. The sub will receive the WebApp object in $_[0], and the session object in $_[1]. Use it as an alternative instead of overriding the C<cs_init()> method when you have just to add some initialization and you don't want to change the default behaviour.
+
+   session_init => 'any_method'
+   session_init => \&any_sub
+   session_init => sub{ ... }
+
 =back
 
 These defaults produce the most compatible and multiplatform CGI::Session object which is equal to:
@@ -193,7 +210,12 @@ These defaults produce the most compatible and multiplatform CGI::Session object
                        { Directory => File::Spec->tmpdir } # DSN_param for File
                      );
 
-B<Note>: You should change just the argument that you need to customize, i.e. usually just DSN and DSN_param.
+B<Note>: You should change just the argument that you need to customize, i.e. usually just DSN and DSN_param and maybe new_session_init:
+
+   $s->cs_new_args( DSN => .... ,
+                    DSN_param => .... ,
+                    new_session_init => sub{ ... },
+                  );
 
 =head1 METHODS
 
@@ -203,21 +225,15 @@ This method is not intended to be used directly in your CBB. It is used internal
 
 =head2 cs_init
 
-This method is internally called by the C<cs_new()> method, so you don't need to use it unless you need to override it. The purpose of this method is setting automatically the session id that has to be sent to the client. It does so by adding the session id cookie to the headers, and by adding also the session id query param to the query.
+This method is internally called by the C<cs_new()> method, so you don't need to know about it unless you need to override it. The purpose of this method is adding the session id cookie to the headers. Besides it adds also the session id query param to the query. (B<Note>: those operations are done only if the session is new and using the $CGI::Session::NAME as the name of the cookie and query param, so you can set that variable as usual to set a different session id name.)
 
-B<Note>: these operations are done only if the session is new and using the $CGI::Session::NAME as the name of the cookie and query param, so you can set that variable as usual to set a different session id name.
-
-You can override this method if you need to execute some code when the C<cs> object is created or if you need to pass the new session id by any other means (in this case you should also accordingly update the 'SID' argument of the C<cs_new_args>).
+You should override this method only if you need to pass the new session id by any other mean (in this case you should also accordingly update the 'SID' argument of the C<cs_new_args>).
 
 B<Overriding Note>: At the moment this method is executed, the C<cs> property is not set yet, but the CGI::Session object is already created and available as $_[1].
 
 =head1 SUPPORT
 
-Support for all the modules of the CBF is via the mailing list. The list is used for general support on the use of the CBF, announcements, bug reports, patches, suggestions for improvements or new features. The API to the CBF is stable, but if you use the CBF in a production environment, it's probably a good idea to keep a watch on the list.
-
-You can join the CBF mailing list at this url:
-
-L<http://lists.sourceforge.net/lists/listinfo/cgi-builder-users>
+See L<CGI::Builder/"SUPPORT">.
 
 =head1 AUTHOR and COPYRIGHT
 
